#include <avr/wdt.h>
#include <avr/interrupt.h>  /*  sei() */
#include <util/delay.h>     /*  _delay_ms() */ 
#include <avr/pgmspace.h>   /*   usbdrv.h */
#include "usbdrv.h"
#include "settings.h"
#include "pins.h"

u16 timecnt1, timecnt2;
u16 timeout;
u8 	mode;
u32 flashcnt = 0;
u32 ledstate;

struct _recieved
{
	u32 changes;
	u8  value;
} recieved;

void LedControl (u32 changes, u8 value)
//  22   changes   P22..P1.
//  changes  ,   (  value).
{
	u8 A, B, C, D;
	
	B = 0;
	C = 0;
	D = 0;	
	A = (u8)changes & AMASK;
	changes>>=8;
	if (changes & 1)
	{
		B |= (1<<LED09);
	}
	changes>>=1;
	if (changes & 1)
	{
		B |= (1<<LED10);
	}
	changes>>=1;
	if (changes & 1)
	{
		B |= (1<<LED11);
	}
	changes>>=1;
	if (changes & 1)
	{
		B |= (1<<LED12);
	}
	changes>>=1;
	if (changes & 1)
	{
		C |= (1<<LED13);
	}
	changes>>=1;
	if (changes & 1)
	{
		C |= (1<<LED14);
	}
	changes>>=1;
	if (changes & 1)
	{
		C |= (1<<LED15);
	}
	changes>>=1;
	if (changes & 1)
	{
		C |= (1<<LED16);
	}
	changes>>=1;
	if (changes & 1)
	{
		D |= (1<<LED17);
	}
	changes>>=1;
	if (changes & 1)
	{
		D |= (1<<LED18);
	}
	changes>>=1;
	if (changes & 1)
	{
		D |= (1<<LED19);
	}
	changes>>=1;
	if (changes & 1)
	{
		D |= (1<<LED20);
	}
	changes>>=1;
	if (changes & 1)
	{
		D |= (1<<LED21);
	}
	changes>>=1;
	if (changes & 1)
	{
		D |= (1<<LED22);
	}

	if (value)
	{
		PORTA |=  A;
		PORTB |=  B;
		PORTC |=  C;
		PORTD |=  D;
	}
	else
	{
		PORTA &= ~A;
		PORTB &= ~B;
		PORTC &= ~C;
		PORTD &= ~D;
	}
}

/* ------------------------------------------------------------------------- */
/* -----------------------------  USB ----------------------------- */
/* ------------------------------------------------------------------------- */

PROGMEM const char usbHidReportDescriptor[22] = {    /*   USB */
    0x06, 0x00, 0xff,              // USAGE_PAGE (Generic Desktop)
    0x09, 0x01,                    // USAGE (Vendor Usage 1)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x95, 0x80,                    //   REPORT_COUNT (128)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
    0xc0                           // END_COLLECTION
};
/*      feature-,     
 *  report-ID (      ).     128
 *  opaque  .
 */
 
/*        */
static uchar    currentAddress;
static uchar    bytesRemaining; 

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;

    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /*  HID class */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
            /*       ,    -ID */
            bytesRemaining = 128;
            currentAddress = 0;
            return USB_NO_MSG;  /*  usbFunctionRead()       */
        }else if(rq->bRequest == USBRQ_HID_SET_REPORT){
            /*       ,    -ID */
            bytesRemaining = 128;
            currentAddress = 0;
            return USB_NO_MSG;  /* use usbFunctionWrite()       */
        }
    }else{
        /*    ,       */
    }
    return 0;
} 

/* usbFunctionRead() ,       .
 *     .   usbdrv/usbdrv.h.
 */
uchar   usbFunctionRead(uchar *data, uchar len)
{
	u8 cnt;
	u8* pnt;

	ON_LED();
	if(len > bytesRemaining)
        len = bytesRemaining;
    bytesRemaining -= len;
    //    4     ledstate
	pnt = (u8*)&ledstate;
	cnt = len;
	while(cnt)
	{
		if (currentAddress < sizeof (ledstate))
		{
			*data++ = *pnt++;
		}
		else
		{
			//   
			*data++ = 0;
		}
		currentAddress++;
		cnt--;
	}
    return len;
}

/* usbFunctionWrite() ,       .
 *     .   usbdrv/usbdrv.h.
 */
uchar   usbFunctionWrite(uchar *data, uchar len)
{
	u8* pnt;

    if(bytesRemaining == 0)
        return 1;               /*   */
	ON_LED();
    if(len > bytesRemaining)
        len = bytesRemaining;
    bytesRemaining -= len;
	//  5     recieved
	pnt = (u8*)&recieved;
	while(len)
	{
		if (currentAddress < sizeof (recieved))
		{
			pnt[currentAddress] = data[currentAddress];
		}
		//  
		currentAddress++;
		len--;
	}
	if (bytesRemaining == 0)
	{
		if (recieved.value)
		{
			ledstate |= recieved.changes;
		}
		else
		{
			ledstate &= ~recieved.changes;
		}
		//  22 ,  22  P1..P22
		ledstate &= 0x003FFFFF;			//00000000 00111111 11111111 11111111
		LedControl (recieved.changes, recieved.value);
	}
    return bytesRemaining == 0; /*  1,      */
}
 
void portsInit (void)
{
	//    ( 22 ,   22 )
	//8 :
	DDRA  = AMASK;
	//4 
	DDRB  = BMASK;
	//4 
	DDRC  = CMASK;
	//6 
	DDRD  = DMASK;
}

int main (void)
{
	uchar i;
	mode = MODE_RESTART;

	//   EEPROM
	while (1)
	{
		if (MODE_RESTART == mode)
		{
/********************
 *             *
 ********************/
		    wdt_enable(WDTO_1S);
			portsInit();
    		usbInit();
    		usbDeviceDisconnect();  /*   ,  ,   ! */
    		i = 0;
    		while(--i)
			{/*  USB    > 250  */
        		_delay_ms(1);
    		}
    		usbDeviceConnect();
    		sei();
			timecnt1 = 0;
			mode = MODE_MAIN;
		}
        wdt_reset();
        usbPoll();
		flashcnt++;
		flashcnt &= FLASH_PERIOD-1;
		OFF_LED();
	}
	return 0;
}
